# QA Launchpad Bug Reports
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import json
import re

from django.shortcuts import render

from qlbr import TZ
from qlbr.models import Task


def index(request):

    tasks = {}
    for task in Task.objects.iterator():
        bugid = task.bug.bug_id
        report_task = {
            'title': task.bug.title,
            'taglist': task.bug.tag_list,
            'date_created': task.date_created.isoformat(),
            'age': (datetime.datetime.now(TZ) - task.date_created).days,
            'importance': task.importance,
            'status': task.status,
            'web_link': task.web_link,
            'duplicate_link': '',
            'owner': task.owner.display_name,
            'triaged': task.triaged
        }
        dupe_of = task.bug.duplicate_of
        if dupe_of:
            report_task['duplicate_link'] = ('<a href="{}">{}</a>'
                                             .format(task.web_link, bugid))
            report_task['owner'] += ' (dup)'
            bugid = dupe_of.bug_id
            report_task['web_link'] = dupe_of.web_link
            report_task['triaged'] = dupe_of.triaged

        # Pilfered from collect-bug-data in lp:arsenal
        m = re.search('(\S+)\s\(.*\)', task.bug_target_name)
        if m is not None:
            report_task['target'] = m.group(1)
        else:
            report_task['target'] = task.bug_target_name
        m = re.search('(\S+)\s\(.*\)', task.bug_target_display_name)
        if m is not None:
            report_task['target_display_name'] = m.group(1)
        else:
            report_task['target_display_name'] = task.bug_target_display_name
    
        report_task['target'] = report_task['target'].replace('.', '')
        if task.assignee:
            report_task['assignee'] = task.assignee.display_name
        else:
            report_task['assignee'] = 'Unassigned'
        if bugid not in tasks:
            tasks[bugid] = []
        tasks[bugid].append(report_task)

    targets = {}
    for bugid in tasks:
        for task in tasks[bugid]:
            target = task['target']
            name = task['target_display_name']
            if target in targets:
                if target != name:
                    targets[target] = name
            else:
                targets[target] = name
    target_report_order = sorted([(targets[target], target) for target in targets])
    tasks_json_dump = json.dumps(tasks, sort_keys=True, indent=4)
    context = {'report_title': 'Test',
               'tasks': tasks,
               'timestamp': datetime.datetime.now().isoformat(),
               'datefilter': True,
               'target_report_order': target_report_order,
               'tasks_json_dump': tasks_json_dump}
    return render(request, 'qlbr/qa-default.html', context)
